import axios from "axios";

import { Error } from "../util/Error";
// import DistributorJSON from "../../json/distributors.json";
// import ProductJSON from "../../json/products.json";
import {
  DistributorEntity,
  DistributorProduct,
  DistributorGetRequest,
  DistributorsGetRequest,
  DistributorProductsGetRequest,
} from "./Distributor.api.types";
import { Region } from "../ProcurementTool/ProcurementTool.api.types";
import { getAllRegions } from "../../apiEngine";
import { BASE_API_URL } from "../../constants";

export interface IDistributorHandler {
  get(request: DistributorGetRequest): Promise<DistributorEntity>;
  getAll(request: DistributorsGetRequest): Promise<DistributorEntity[]>;
  getAllWithProducts(
    request: DistributorsGetRequest
  ): Promise<DistributorEntity[]>;
  getAllWithProductsToOrder(
    request: DistributorsGetRequest
  ): Promise<DistributorEntity[]>;

  getExtraData(request: DistributorGetRequest): Promise<DistributorEntity>;
  getAllExtraData(regionId: number): Promise<DistributorEntity[]>;

  getAllComplete(request: DistributorsGetRequest): Promise<DistributorEntity[]>; //Merge getAll and GetAllExtraData

  getProducts(
    request: DistributorProductsGetRequest
  ): Promise<DistributorProduct[]>;
  getProductsWithNonZeroOrderQuantity(
    request: DistributorProductsGetRequest
  ): Promise<DistributorProduct[]>;

  getAllRegions(): Promise<Region[]>;
}

export class DistributorHandler implements IDistributorHandler {
  async get(request: DistributorGetRequest): Promise<DistributorEntity> {
    const url = `${BASE_API_URL}/distributor/${request.id}`;
    let response;
    try {
      response = await axios.get(url);

      return new DistributorEntity(response.data[0]);
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching a distributor. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
  async getAll(request: DistributorsGetRequest): Promise<DistributorEntity[]> {
    const url = `${BASE_API_URL}/distributors/${request.regionId}`;
    let response;
    try {
      response = await axios.get(url);
      return response.data.map(
        (distributor: any) => new DistributorEntity(distributor)
      );
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor list. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
  async getAllWithProducts(
    //Todo: use
    request: DistributorsGetRequest
  ): Promise<DistributorEntity[]> {
    const url = `${BASE_API_URL}/distributors/withProducts/${request.regionId}`;
    let response;
    try {
      response = await axios.get(url);
      return response.data.map(
        (distributor: any) => new DistributorEntity(distributor)
      );
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor list(with products). Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
  async getAllWithProductsToOrder(
    //Todo: use
    request: DistributorsGetRequest
  ): Promise<DistributorEntity[]> {
    const url = `${BASE_API_URL}/distributors/withProductsToOrder/${request.regionId}`;
    let response;
    try {
      response = await axios.get(url);
      return response.data.map(
        (distributor: any) => new DistributorEntity(distributor)
      );
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor list(with products to order). Try connecting to the ISU VPN or running backend locally",
      });
    }
  }

  async getExtraData(
    request: DistributorGetRequest
  ): Promise<DistributorEntity> {
    const url = `${BASE_API_URL}/distributor/schedule/${request.id}`;
    let response;
    try {
      response = await axios.get(url);
      return new DistributorEntity(
        response.data.length > 0 ? response.data[0] : {}
      );
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching a distributor. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
  async getAllExtraData(regionId: number): Promise<DistributorEntity[]> {
    const url = `${BASE_API_URL}/distributors/scheduleAll/${regionId}`;
    let response;
    try {
      response = await axios.get(url);
      const result: DistributorEntity[] = response.data.map(
        (distributor: any) => new DistributorEntity(distributor)
      );
      return result;
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor list. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }

  async getProducts(
    request: DistributorProductsGetRequest
  ): Promise<DistributorProduct[]> {
    const url = `${BASE_API_URL}/distributor/products/${request.distributorId}`;
    let response;
    try {
      response = await axios.get(url);

      const products: DistributorProduct[] = response.data.map(
        (product: any, index: number) =>
          new DistributorProduct({ ...product, key: index })
      );

      return products;
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor product list. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }

  async getProductsWithNonZeroOrderQuantity(
    //Todo: Use
    request: DistributorProductsGetRequest
  ): Promise<DistributorProduct[]> {
    const url = `${BASE_API_URL}/distributor/products/withpredictions/${request.distributorId}`;
    let response;
    try {
      response = await axios.get(url);

      const products: DistributorProduct[] = response.data.map(
        (product: any, index: number) =>
          new DistributorProduct({ ...product, key: index })
      );

      return products;
    } catch (error) {
      throw new Error({
        error: error,
        message:
          "There was a problem fetching the distributor product list(WithNonZeroOrderQuantity). Try connecting to the ISU VPN or running backend locally",
      });
    }
  }

  async getAllComplete(
    request: DistributorsGetRequest
  ): Promise<DistributorEntity[]> {
    let incompleteDistributors: DistributorEntity[];
    let incompleteScheduleDistributors: DistributorEntity[];
    try {
      if (request.onlyDistributorsWithProducts) {
        incompleteDistributors = await this.getAllWithProducts(request); //Todo: evaluate this when issue#18 is updated
      } else {
        incompleteDistributors = await this.getAll(request);
      }
      incompleteScheduleDistributors = await this.getAllExtraData(
        request.regionId
      );
    } catch (e) {
      //Could do something here but for now just re-propigate error
      throw e;
    }

    //Merge
    const distributors: DistributorEntity[] = incompleteDistributors.map(
      (distributor: DistributorEntity) => {
        const matchingExtraDataDistributor:
          | DistributorEntity
          | undefined = incompleteScheduleDistributors.find(
          (d: DistributorEntity) => d.id === distributor.id
        );

        const combined = new DistributorEntity(distributor);
        combined.extraData = matchingExtraDataDistributor
          ? matchingExtraDataDistributor.extraData
          : undefined;
        return combined;
      }
    );

    return distributors;
  }

  async getAllRegions(): Promise<Region[]> {
    return await getAllRegions();
  }
}

// export class FakeDistributorHandler implements IDistributorHandler {
//   async getExtraData(
//     request: DistributorGetRequest
//   ): Promise<DistributorEntity> {
//     throw new Error("Method not implemented.");
//   }
//   async get(request: DistributorGetRequest): Promise<DistributorEntity> {
//     return new DistributorEntity(DistributorJSON[0]);
//   }
//   async getAll(request: DistributorsGetRequest): Promise<DistributorEntity[]> {
//     return DistributorJSON.map(
//       (distributor: any) => new DistributorEntity(distributor)
//     );
//   }
//   async getAllExtraData(): Promise<DistributorEntity[]> {
//     return DistributorJSON.map(
//       (distributor: any) => new DistributorEntity(distributor)
//     );
//   }
//   async getAllComplete(): Promise<DistributorEntity[]> {
//     return DistributorJSON.map(
//       (distributor: any) => new DistributorEntity(distributor)
//     );
//   }
//   async getProducts(
//     request: DistributorProductsGetRequest
//   ): Promise<DistributorProduct[]> {
//     return ProductJSON.map(
//       (product: any, index: number) =>
//         new DistributorProduct({ ...product, key: index })
//     );
//   }
// }
