import * as React from "react";
import { Collapse, Descriptions, Button, Tooltip } from "antd";
import { IDistributorHandler } from "../DistributorTable/Distributor.api.handler";
import {
  DistributorEntity,
  DistributorProduct,
  DistributorProductsGetRequest,
} from "../DistributorTable/Distributor.api.types";
import PurchaseOrderTable from "./PurchaseOrderTable";
import { createOrderScheduleString } from "../DistributorTable/DistributorTable";
import CurrencyInput from "../util/CurrencyInput";

const { Panel } = Collapse;
const { Item } = Descriptions;

export interface IDistributorCardProps {
  handler: IDistributorHandler;
  distributor: DistributorEntity;
}

interface IDistributorCardState {
  products: DistributorProduct[] | undefined;
  totalDistributorOrderCost: number | undefined;
  showProductsNoOrder: boolean;
  isOpen: boolean;
}

export default class DistributorCard extends React.Component<
  IDistributorCardProps,
  IDistributorCardState
> {
  state: IDistributorCardState = {
    products: undefined,
    totalDistributorOrderCost: undefined,
    showProductsNoOrder: false,
    isOpen: false,
  };

  onOpenCollapse = async () => {
    const { isOpen, showProductsNoOrder } = this.state;

    if (!isOpen) {
      const products: DistributorProduct[] = await this.props.handler!.getProducts(
        new DistributorProductsGetRequest({
          distributorId: this.props.distributor.id,
        })
      );
      const totalDistributorOrderCost = products
        .map((product: DistributorProduct) => product.purchaseCost)
        .reduce(
          (accumulator: number, currentValue: number) =>
            accumulator + currentValue,
          0
        );

      let productsToShow = products;
      if (!showProductsNoOrder && productsToShow) {
        productsToShow = productsToShow.filter(
          (product: DistributorProduct) => product.qtyToOrder
        );
      }

      this.setState({
        products: productsToShow,
        totalDistributorOrderCost: totalDistributorOrderCost,
        isOpen: !isOpen,
      });
    }
    this.setState({
      isOpen: !isOpen,
    });
  };

  /**Distributor information for the distributor card using implicit return (only one react node)*/
  distributorInformation = (distributor: DistributorEntity) => {
    return (
      <Descriptions
        title={`${distributor.id ? distributor.id : "-"}. ${
          distributor.name ? distributor.name : "-"
        }`}
      >
        <Item label="Order Days">
          {createOrderScheduleString(distributor.orderSchedule)}
        </Item>
        <Item label="Buffer">{distributor.buffer || "-"}</Item>
        <Item label="Order Min">{distributor.orderMin || "-"}</Item>
        <Item label="Term">{distributor.term || "-"}</Item>

        <Item label="Week Basis">
          {distributor.extraData ? distributor.extraData.weekBasis : "-"}
        </Item>
        <Item label="Days To Delivery">
          {distributor.extraData ? distributor.extraData.daysToDelivery : "-"}
        </Item>
        <Item label="Days To Delivery End">
          {distributor.extraData
            ? distributor.extraData.daysToDeliveryEnd
            : "-"}
        </Item>
        {/* <Item label="Start Date">
          {distributor.extraData && distributor.extraData.startDate
            ? distributor.extraData.startDate.format("MMM Do YY")
            : "-"}
        </Item> */}
      </Descriptions>
    );
  };

  public render() {
    const { distributor } = this.props;
    const { showProductsNoOrder, products } = this.state;

    return (
      <Collapse
        style={{ margin: "5px" }}
        defaultActiveKey={undefined}
        onChange={(key: string | string[]) => this.onOpenCollapse()}
      >
        <Panel header={this.distributorInformation(distributor)} key="1">
          <div
            style={{
              display: "flex",
              alignItems: "center",
              justifyContent: "flex-start",
              margin: "5px",
            }}
          >
            <p style={{ margin: "10px" }}>
              {products ? products.length : 0} Products
            </p>

            <Tooltip
              title={`${
                showProductsNoOrder ? "Hide" : "Show"
              } Products where Quantity to Order is 0`}
            >
              <Button
                onClick={() =>
                  this.setState({
                    showProductsNoOrder: !showProductsNoOrder,
                  })
                }
                type={products && products.length > 0 ? "default" : "primary"}
              >
                {`${showProductsNoOrder ? "Hide" : "Show"} Empty Products`}
              </Button>
            </Tooltip>
          </div>
          <PurchaseOrderTable products={products} />
          <div
            style={{
              display: "flex",
              justifyContent: "flex-end",
              marginTop: "10px",
            }}
          >
            <span style={{ margin: "5px" }}>
              Total Distributor Order Cost:{" "}
            </span>
            {CurrencyInput({
              value: this.state.totalDistributorOrderCost,
              disabled: true,
            })}
          </div>
        </Panel>
      </Collapse>
    );
  }
}
