import * as React from "react";
import {
  Spin,
  Result,
  Button,
  Select,
  Checkbox,
  Pagination,
  Input,
  Empty,
  Modal,
} from "antd";
import FuzzySearch from "fuzzy-search";

import {
  DistributorEntity,
  DistributorsGetRequest,
} from "../DistributorTable/Distributor.api.types";
import {
  IDistributorHandler,
  DistributorHandler,
} from "../DistributorTable/Distributor.api.handler";
import { WEEK_DAYS } from "../../constants";
import DistributorCard from "./DistributorCard";
import { Region } from "../ProcurementTool/ProcurementTool.api.types";

/**
author: @lssleeth @jkelley
Distributor Collapsable is used for storing a table of items associated with a distributor
Using Collapsable module from antdesign 
*/

const { Option } = Select;

interface IDistributorCollapsableProps {
  handler?: IDistributorHandler;
}

interface IDistributorCollapsableState {
  distributors: DistributorEntity[];
  error: string;
  selectedFilterDays: string[];
  useDayFilter: boolean;
  currentPageNumber: number;
  searchValue: string;
  regions: Region[];
  selectedRegionId: number | undefined;
  resultsPerPage: number;
  editFiltersModalVisible: boolean;
  useTermFilter: boolean;
  minTerm: number;
  maxTerm: number;
}

const INIT_FILTER_VALUES = {
  useDayFilter: false,
  useTermFilter: false,
  searchValue: "",
  selectedFilterDays: [],
  minTerm: 0,
  maxTerm: 5,
};

/*Create a class which can have both props and state*/
/*Class provides lifecyle methods a.k.a render and others*/
export default class DistributorCollapsable extends React.Component<
  IDistributorCollapsableProps,
  IDistributorCollapsableState
> {
  static defaultProps = {
    handler: new DistributorHandler(),
  };
  state: IDistributorCollapsableState = {
    distributors: [],
    error: "",
    currentPageNumber: 1,
    regions: [],
    selectedRegionId: undefined,
    resultsPerPage: 10,
    editFiltersModalVisible: false,
    ...INIT_FILTER_VALUES,
  };
  componentDidMount = async () => {
    const regionId = await this.fetchRegions();
    this.fetchDistributors(regionId);
  };

  fetchRegions = async (): Promise<number | undefined> => {
    const { selectedRegionId } = this.state;

    let regionEntities: Region[];
    try {
      regionEntities = await this.props.handler!.getAllRegions();
      let regionId: number | undefined = undefined;
      if (regionEntities.length > 0) {
        regionId = selectedRegionId ? selectedRegionId : regionEntities[0].id;
      }
      this.setState({
        regions: regionEntities,
        selectedRegionId: regionId,
      });
      return regionId;
    } catch (e) {
      this.setState({
        error: e.message,
      });
    }
  };

  handleRegionChange = (val: number) => {
    //Update distributors
    this.fetchDistributors(val);
    this.setState({
      currentPageNumber: 1,
      selectedRegionId: val,
    });
  };

  fetchDistributors = async (regionId: number | undefined) => {
    //Fetch all distributors
    this.setState({
      error: "",
    });
    let distributors: DistributorEntity[];
    try {
      distributors = await this.props.handler!.getAllComplete(
        new DistributorsGetRequest({
          districtId: regionId ? regionId : 1,
          onlyDistributorsWithProducts: true,
        })
      );
      this.setState({
        distributors: distributors,
      });
    } catch (e) {
      this.setState({
        error: e.message,
      });
    }
  };

  handleselectedFilterDaysChange = (selectedFilterDays: string[]) => {
    this.setState({ selectedFilterDays });
  };

  passesUserFilters = (distributorItem: DistributorEntity) => {
    const {
      selectedFilterDays,
      useDayFilter,
      useTermFilter,
      minTerm,
      maxTerm,
    } = this.state;

    const { orderSchedule, term } = distributorItem;

    const selectedFilterDaysObj = {
      monday: selectedFilterDays.includes(WEEK_DAYS[0]),
      tuesday: selectedFilterDays.includes(WEEK_DAYS[1]),
      wednesday: selectedFilterDays.includes(WEEK_DAYS[2]),
      thursday: selectedFilterDays.includes(WEEK_DAYS[3]),
      friday: selectedFilterDays.includes(WEEK_DAYS[4]),
      saturday: selectedFilterDays.includes(WEEK_DAYS[5]),
      sunday: selectedFilterDays.includes(WEEK_DAYS[6]),
    };

    if (useDayFilter) {
      if (
        !orderSchedule.monday &&
        !orderSchedule.tuesday &&
        !orderSchedule.wednesday &&
        !orderSchedule.thursday &&
        !orderSchedule.friday &&
        !orderSchedule.saturday &&
        !orderSchedule.sunday
      ) {
        return false;
      }

      if (selectedFilterDaysObj.monday && !orderSchedule.monday) return false;
      if (selectedFilterDaysObj.tuesday && !orderSchedule.tuesday) return false;
      if (selectedFilterDaysObj.wednesday && !orderSchedule.wednesday)
        return false;
      if (selectedFilterDaysObj.thursday && !orderSchedule.thursday)
        return false;
      if (selectedFilterDaysObj.friday && !orderSchedule.friday) return false;
      if (selectedFilterDaysObj.saturday && !orderSchedule.saturday)
        return false;
      if (selectedFilterDaysObj.sunday && !orderSchedule.sunday) return false;
    }

    if (useTermFilter) {
      if (!term) return false;
      if (term < minTerm || term > maxTerm) return false;
    }

    return true;
  };

  onPaginationChange = (page: number, pageSize?: number | undefined) => {
    this.setState({
      currentPageNumber: page,
    });
  };

  onSearchValueChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    this.setState({
      searchValue: event.target.value,
      currentPageNumber: 1,
    });
  };

  onPaginationShowSizeChange = (current: number, size: number) => {
    this.setState({
      currentPageNumber: current,
      resultsPerPage: size,
    });
  };

  render() {
    const {
      error,
      distributors,
      selectedFilterDays,
      useDayFilter,
      currentPageNumber,
      searchValue,
      selectedRegionId,
      regions,
      resultsPerPage,
      useTermFilter,
      minTerm,
      maxTerm,
    } = this.state;

    if (error) {
      return (
        <Result
          status="warning"
          title={error}
          extra={
            <Button
              type="primary"
              onClick={() => this.fetchDistributors(selectedRegionId)}
            >
              Reload
            </Button>
          }
        />
      );
    }
    //== evaluates whether they're equal, === whether they're equal and their types are equal
    if (!distributors || distributors.length === 0) {
      return (
        <div
          className="test"
          style={{
            height: "100%",
            display: "flex",
            flexDirection: "column",
            justifyContent: "center",
            alignItems: "center",
          }}
        >
          <h1>Loading...</h1>
          <Spin size="large" />
        </div>
      );
    }
    const filteredOptions = WEEK_DAYS.filter(
      (day: any, i: number) => !selectedFilterDays.includes(day)
    );

    //This way of filtering here seems like bad practice since a new FuzzySearch is being constructed every re-render
    //Alt1: Keep a display list of distributors in the state that has been already filtered
    //Con: Need to be careful to update it correctly whenever state changes (could use another lifecycle method to run search again? [ComponentDidUpdate?])

    const filteredDistributors: DistributorEntity[] = distributors.filter(
      this.passesUserFilters
    );
    const searcher = new FuzzySearch(filteredDistributors, ["name", "id"], {
      caseSensitive: false,
    });
    let searchedDistributors: DistributorEntity[] = filteredDistributors;
    if (searchValue !== "") {
      searchedDistributors = searcher.search(searchValue);
    }
    return (
      <div>
        <div
          className="filters"
          style={{
            display: "flex",
            flexDirection: "column",
            alignItems: "space-around",
          }}
        >
          <div
            style={{
              display: "flex",
              justifyContent: "space-around",
            }}
          >
            <Select
              style={{ width: 120 }}
              value={
                regions.length > 0
                  ? this.state.selectedRegionId || regions[0].id
                  : 1
              }
              onChange={this.handleRegionChange}
              filterOption={(input, option: any) =>
                option.props.children
                  .toLowerCase()
                  .indexOf(input.toLowerCase()) >= 0
              }
              optionFilterProp="children"
              showSearch
            >
              {regions.map((region: Region, i: number) => (
                <Option value={region.id} key={i}>
                  {region.name}
                </Option>
              ))}
            </Select>
            <Input
              value={searchValue}
              onChange={(event: React.ChangeEvent<HTMLInputElement>) =>
                this.onSearchValueChange(event)
              }
              placeholder="Search for distributor name/id"
              style={{ width: "40%", maxWidth: "80%", minWidth: "100px" }}
            />
            <p>
              Showing:{" "}
              {searchedDistributors.length > 0
                ? Math.min(
                    resultsPerPage,
                    currentPageNumber ===
                      Math.ceil(searchedDistributors.length / resultsPerPage)
                      ? searchedDistributors.length % resultsPerPage
                      : resultsPerPage
                  )
                : 0}{" "}
              / {searchedDistributors.length} Distributors
            </p>
            <Button
              type="primary"
              onClick={() => this.setState({ editFiltersModalVisible: true })}
            >
              Edit Filters
            </Button>
            <Button
              type="danger"
              onClick={() => this.setState({ ...INIT_FILTER_VALUES })}
            >
              Clear Filters
            </Button>
          </div>
        </div>
        <div
          style={{
            display: "flex",
            flexDirection: "column",
            padding: "5px",
            marginBottom: "100px",
          }}
        >
          <>
            {searchedDistributors.length > 0 ? (
              searchedDistributors
                .slice(
                  resultsPerPage * (currentPageNumber - 1),
                  resultsPerPage * (currentPageNumber - 1) + resultsPerPage
                )
                .map((distributorItem: DistributorEntity, i: number) => {
                  return (
                    <DistributorCard
                      key={i}
                      handler={this.props.handler!}
                      distributor={distributorItem}
                    />
                  );
                })
            ) : (
              <Empty />
            )}

            <div
              style={{
                width: "100%",
                display: "flex",
                justifyContent: "center",
                alignItems: "center",
                paddingTop: "20px",
              }}
            >
              <Pagination
                current={currentPageNumber}
                defaultPageSize={resultsPerPage}
                showSizeChanger
                onShowSizeChange={this.onPaginationShowSizeChange}
                total={searchedDistributors.length}
                onChange={this.onPaginationChange}
              />
            </div>
          </>
        </div>
        <Modal
          title="Additional Distributor Filters"
          visible={this.state.editFiltersModalVisible}
          onCancel={() => {
            this.setState({ editFiltersModalVisible: false });
          }}
          footer={[
            <Button
              key="ok"
              type="primary"
              onClick={() => {
                this.setState({ editFiltersModalVisible: false });
              }}
            >
              Ok
            </Button>,
          ]}
        >
          <div
            style={{
              display: "flex",
              justifyContent: "flex-start",
              alignItems: "center",
              marginLeft: "60px",
              marginTop: "20px",
            }}
          >
            <Checkbox
              checked={useDayFilter}
              onChange={(e) => {
                this.setState({
                  useDayFilter: e.target.checked,
                });
              }}
            >
              Filter by Order Day
            </Checkbox>
            <Select
              mode="multiple"
              disabled={!useDayFilter}
              placeholder="Filter by order day"
              value={selectedFilterDays}
              onChange={this.handleselectedFilterDaysChange}
              style={{ minWidth: "175px" }}
            >
              {filteredOptions.map((day: any, i: number) => {
                return (
                  <Option key={i} value={day}>
                    {day}
                  </Option>
                );
              })}
            </Select>
          </div>

          <div
            style={{
              display: "flex",
              justifyContent: "flex-start",
              alignItems: "center",
              marginLeft: "60px",
              marginTop: "20px",
            }}
          >
            <Checkbox
              checked={useTermFilter}
              onChange={(e) => {
                this.setState({
                  useTermFilter: e.target.checked,
                });
              }}
            >
              Filter by Term
            </Checkbox>
            <div>
              Min:{" "}
              <Input
                type="number"
                value={minTerm}
                min={0}
                onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
                  this.setState({
                    minTerm: parseInt(event.target.value),
                    maxTerm: Math.max(parseInt(event.target.value), maxTerm),
                  });
                }}
                disabled={!useTermFilter}
                style={{ width: "100px" }}
              />
            </div>

            <div>
              Max:{" "}
              <Input
                type="number"
                value={maxTerm}
                min={0}
                onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
                  this.setState({
                    maxTerm: Math.max(parseInt(event.target.value), minTerm),
                  });
                }}
                disabled={!useTermFilter}
                style={{ width: "100px" }}
              />
            </div>
          </div>
          {/* <div>
            <Checkbox
              checked={showEmptyDistributors}
              onChange={(e) => {
                this.setState({
                  showEmptyDistributors: e.target.checked,
                });
              }}
            >
              Hide Distributors with no needed orders
            </Checkbox>
          </div> */}
        </Modal>
      </div>
    );
  }
}
